/* 
 * Copyright (C) 2013  Franck van Breugel
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You can find a copy of the GNU General Public License at
 * <http://www.gnu.org/licenses/>.
 */

package probabilistic.listener;

import java.io.Serializable;

/**
 * This class represent a transition.  Each transition has a source and
 * target state and a probability.
 *
 * @author Franck van Breugel
 */
public class Transition implements Serializable 
{
    private int source;
    private int target;
    private double probability;

    /**
     * Intializes this transition with the given source, target and
     * probability.
     *
     * @param source the source state of this transition.
     * @param target the target state of this transition.
     * @param probability the probabilty of this transition.
     * @pre. probability > 0 && probability <= 1
     */
    public Transition(int source, int target, double probability)
    {
	this.source = source;
	this.target = target;
	this.probability = probability;
    }

    /**
     * Returns the source of this transition.
     *
     * @return the source of this transition.
     */
    public int getSource()
    {
	return source;
    }

    /**
     * Returns the target of this transition.
     *
     * @return the target of this transition.
     */
    public int getTarget()
    {
	return target;
    }

    /**
     * Returns the probability of this transition.
     *
     * @return the probability of this transition.
     */
    public double getProbability()
    {
	return probability;
    }

    /**
     * Returns a string representation of this transition.
     *
     * @return a string representation of this transition.
     */
    public String toString()
    {
	return source + " -" + probability + "-> " + target;
    } 
}
