/* 
 * Copyright (C) 2011  Xin Zhang and Franck van Breugel
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You can find a copy of the GNU General Public License at
 * <http://www.gnu.org/licenses/>.
 */

package probabilistic.vm;

import java.util.Arrays;
import gov.nasa.jpf.vm.choice.IntIntervalGenerator;

/**
 * A choice generator for the interval <code>[0, p.length - 1]</code> which
 * keeps also track of the probabilities of the choices.  Choice <code>i</code>
 * has probability <code>p[i]</code>.
 *
 * @author Xin Zhang
 * @author Franck van Breugel
 */
public class ProbabilisticChoiceGenerator extends IntIntervalGenerator implements Probabilistic
{
    private double[] p;
  
    /**
     * Initializes a ProbabilisticChoiceGenerator with the given id and probabilities.
     *
     * @param id id.
     * @param p probabilities.
     * @pre. <code>p[0] + ... + p[p.length - 1] = 1.0</code>.
     */
    public ProbabilisticChoiceGenerator (String id, double[] p) 
    {
	super(id, 0, p.length - 1); 
	/*
	 * The array p is copied so that modifications of the array by the 
	 * application do not interfere with JPF.
	 */
	this.p = Arrays.copyOf(p, p.length);
    }
    
    /**
     * Returns the probability of the current choice.
     *
     * @return the probability of the current choice.
     */
    public double getProbability() 
    {
	return this.p[this.next];
    }
}
